import "./main";
import CTFd from "../compat/CTFd";
import $ from "jquery";
import { htmlEntities } from "@ctfdio/ctfd-js/utils/html";
import { ezQuery, ezAlert } from "../compat/ezq";
import { showProvided } from "../compat/visual.js";
import "../compat/format";

window.carouselPosition = 0;
window.carouselMax = 0;

function deleteCorrectSubmission(_event) {
  const key_id = $(this).data("submission-id");
  const $elem = $(this).parent().parent();
  const chal_name = $elem.find(".chal").text().trim();
  const team_name = $elem.find(".team").text().trim();

  const row = $(this).parent().parent();

  ezQuery({
    title: "Supprimer soumission",
    body: "Êtes-vous certain de vouloir supprimer la soumission de l'équipe {0} pour le défi {1}".format(
      "<strong>" + htmlEntities(team_name) + "</strong>",
      "<strong>" + htmlEntities(chal_name) + "</strong>"
    ),
    success: function () {
      alert({ submissionId: key_id });
      CTFd.api
        .delete_submission({ submissionId: key_id })
        .then(function (response) {
          if (response.success) {
            row.remove();
          }
        });
    },
  });
}

function deleteSelectedSubmissions(_event) {
  let submissionIDs = $("input[data-submission-id]:checked").map(function () {
    return $(this).data("submission-id");
  });
  let target = submissionIDs.length === 1 ? "submission" : "submissions";

  ezQuery({
    title: "Supprimer une soumission",
    body: `Êtes-vous certain de vouloir supprimer ${submissionIDs.length} ${target}?`,
    success: function () {
      const reqs = [];

      for (var subId of submissionIDs) {
        reqs.push(CTFd.api.delete_submission({ submissionId: subId }));
      }
      Promise.all(reqs).then((_responses) => {
        window.location.reload();
      });
    },
  });
}

function correctSubmissions(_event) {
  let submissionIDs = $("input[data-submission-id]:checked").map(function () {
    return $(this).data("submission-id");
  }).get();
  
  let target = submissionIDs.length === 1 ? "submission" : "submissions";
  let validSubmissions = [];

  ezQuery({
    title: "Valider une soumission",
    body: `Êtes-vous sûr de vouloir marquer ${submissionIDs.length} ${target} correct?`,
    success: function () {
      const checkPromises = submissionIDs.map(subId => {
        return CTFd.fetch(`/api/v1/submissions/${subId}`, {
          method: "GET",
          credentials: "same-origin",
          headers: {
            Accept: "application/json",
          },
        }).then(response => response.json()).then(data => {
          if (data.data.provided !== "Média en traitement") {
            validSubmissions.push(subId);
          } else {
            ezAlert({
              title: "Erreur",
              body: `Soumission ID ${subId} ne peut être marquée comme correcte car le media est encore en traitement.`,
              button: "OK"
            });
          }
        });
      });

      Promise.all(checkPromises).then(() => {
        if (validSubmissions.length > 0) {
          const reqs = validSubmissions.map(subId => {
            return CTFd.fetch(`/api/v1/submissions/${subId}`, {
              method: "PATCH",
              credentials: "same-origin",
              headers: {
                Accept: "application/json",
                "Content-Type": "application/json",
              },
              body: JSON.stringify({ type: "correct" }),
            });
          });

          Promise.all(reqs).then(() => {
            window.location.reload();
          });
        }
      });
    },
  });
}

function showFlagsToggle(_event) {
  const urlParams = new URLSearchParams(window.location.search);
  if (urlParams.has("full")) {
    urlParams.delete("full");
  } else {
    urlParams.set("full", "true");
  }
  window.location.href = `${window.location.pathname}?${urlParams.toString()}`;
}

function showFlag(event) {
  let target = $(event.currentTarget);
  let eye = target.find("i");
  let flag = target.parent().find("pre");
  if (!flag.hasClass("full-flag")) {
    flag.text(flag.attr("title"));
    flag.addClass("full-flag");
    eye.addClass("fa-eye-slash");
    eye.removeClass("fa-eye");
  } else {
    flag.text(flag.attr("title").substring(0, 42) + "...");
    flag.removeClass("full-flag");
    eye.addClass("fa-eye");
    eye.removeClass("fa-eye-slash");
  }
}

function copyFlag(event) {
  let target = $(event.currentTarget);
  let flag = target.parent().find("pre");
  let text = flag.attr("title");
  navigator.clipboard.writeText(text);

  $(event.currentTarget).tooltip({
    title: "Copié!",
    trigger: "manual",
  });
  $(event.currentTarget).tooltip("show");

  setTimeout(function () {
    $(event.currentTarget).tooltip("hide");
  }, 1500);
}

function updateValue(event) {
  const target = $(event.target).closest("[data-submission-id]"); // Look for the closest parent with data attribute
  const submissionId = target.data("submission-id");

  if (!submissionId) {
    alert("Submission ID is not defined.");
    return;
  }
  const currentValue = $(event.target).text(); // Get the current text value
  const newValue = prompt("Enter the new value for the submission:", currentValue); // Prompt for a new value

  if (newValue !== null && newValue.trim() !== "") {
      // Send a PATCH request to update the submission value
      CTFd.fetch(`/api/v1/submissions/${submissionId}`, {
        method: "PATCH",
        credentials: "same-origin",
        headers: {
        Accept: "application/json",
        "Content-Type": "application/json",
        },
        body: JSON.stringify({ value: newValue.trim() }),
      }).then(window.location.reload());
  } else {
      alert("Invalid input. Update canceled.");
  }
}

$(() => {
  $("#show-full-flags-button").click(showFlagsToggle);
  $("#show-short-flags-button").click(showFlagsToggle);
  $(".show-flag").click(showFlag);
  $(".copy-flag").click(copyFlag);
  $("#correct-flags-button").click(correctSubmissions);
  $(".delete-correct-submission").click(deleteCorrectSubmission);
  $("#submission-delete-button").click(deleteSelectedSubmissions);
  $(".submission-value").click(updateValue);
});
let elements = document.getElementsByClassName("imageContainer");
for (let i = 0; i < elements.length; i++) {
  showProvided(elements[i]);
}
